# BigQueryUtils.py
from google.cloud import bigquery
import google.auth
from enum import Enum
import pandas as pd
from fp_th_di.logger import logger

class BigQuery(Enum):
  def create_connection(self, projectId:str): 
    """Create Google bigquery client 

    Args:
        projectId (str): GCP project id to run BigQuery job on

    Raises:
        e: Exception
    """    
    credentials,project = google.auth.default(scopes=[
      'https://www.googleapis.com/auth/drive',
      'https://www.googleapis.com/auth/bigquery',
    ])
    self.client = None
    try:
      projectId = self.value
      self.client = bigquery.Client(project=projectId, credentials=credentials)
    except Exception as e:
      logger.error(e)
      raise e

  def fetch(self, statement:str) -> bigquery.QueryJob:
    """Fetch data from BigQuery

    Args:
        statement (str): bigquery sql statement to execute

    Raises:
        Exception: if failed to query data from bigquery

    Returns:
        bigquery.QueryJob: querying result
    """    
    if self.client is None:
      raise Exception('Client not found. Please call create_connection().')

    try: 
      return self.client.query(statement) 
    except Exception as e:
      logger.error(e)
      raise e
  
  def fetch_dataframe(self, statement:str) -> pd.DataFrame:
    """Fetch data from BigQuery as pandas.DataFrame

    Args:
        statement (str): bigquery sql statement to execute

    Raises:
        Exception: if failed to query data from bigquery

    Returns:
        pd.DataFrame: pandas.DataFrame with querying result
    """    
    if self.client is None:
      raise Exception('Client not found. Please call create_connection().')

    try:
      return self.fetch(statement).to_dataframe() 
    except Exception as e:
      logger.error(e)
      raise e

  def execute(self, statement:str) -> None:
    """Execute statement in BigQuery

    Args:
        statement (str): bigquery sql statement to execute

    Raises:
        Exception: if failed to query data from bigquery
    """    
    if self.client is None:
      raise Exception('Client not found. Please call create_connection().')

    try:
      queryJob = self.client.query(statement)  
      queryJob.result()  # Waits for statement to finish
      logger.info("Query completed successfully")
    except Exception as e:
      logger.error(e)
      raise e
  
  def create_table_in_bq(self, project:str, datset:str, table_name:str, schema:list, partition_by_column:str=None) -> None:
    """Create new table in BigQuery

    Args:
        project (str): project name
        datset (str): dataset name
        table_name (str): table name
        schema (list): table schema
        partition_by_column (str): partition column 
    """    
    dataset_ref = bigquery.DatasetReference(project, datset)

    table_ref = dataset_ref.table(table_name)
    table = bigquery.Table(table_ref, schema=schema)
    if partition_by_column is not None:
      table.time_partitioning = bigquery.TimePartitioning(
          type_=bigquery.TimePartitioningType.DAY,
          field=partition_by_column, 
      ) 

    table = self.client.create_table(table)

    logger.info(
        "Created table {}, partitioned on column {}".format(
            table.table_id, table.time_partitioning.field if partition_by_column is not None else 'None'
        )
    )

  def upload_dataframe_to_bq(self, targetedTable:str, dataframe:pd.DataFrame, schema:list, writeDisposition:str='WRITE_APPEND') -> None:
    """
      Args:
        targetedTable (str) : targeted table id (full table name)
        schema (list) : table schema
        dataframe (pd.DataFrame) : pandas dataframe to import to targeted table
        writeDisposition (str) : Optionally, specifies the action that occurs if the destination table already exists.
            The following values are supported:
              WRITE_TRUNCATE: If the table already exists, BigQuery overwrites the table data and uses the schema from the load.
              WRITE_APPEND: If the table already exists, BigQuery appends the data to the table.
              WRITE_EMPTY: If the table already exists and contains data, a 'duplicate' error is returned in the job result.
            The default value is WRITE_APPEND.
    """
    try:
      jobConfig = bigquery.LoadJobConfig(
        schema=schema,
        write_disposition=writeDisposition
      )
      
      # Make an API request.
      job = self.client.load_table_from_dataframe(
          dataframe, targetedTable, job_config=jobConfig
      )  
      job.result()  # Wait for the job to complete.
    except Exception as e:
      logger.error(e)
      raise e


#!/usr/bin/env python3.8
# -*- coding: utf-8 -*-

"""This script contains helper functions related to date and datetime
"""

import datetime as dt
import os

HOUR_OFFSET_ENV_NAME='HOUR_OFFSET'

def set_hour_offset(offset:float):
  """Set offset hour to add to datetime.now timestamp

  Args:
      offset (float): 
  """  
  os.environ[HOUR_OFFSET_ENV_NAME] = offset

def get_hour_offset(offset:float):
  """Get offset hour set for your environment
  """
  return os.getenv(HOUR_OFFSET_ENV_NAME, None)

def now() -> dt.datetime:
  """Gets current datetime.
    To add hour offset, set HOUR_OFFSET environment using os.setenv function

  Returns:
      dt.datetime: datetime object
  """    
  return (dt.datetime.now() + dt.timedelta(hours=os.getenv(HOUR_OFFSET_ENV_NAME, 0)))

def today() -> dt.date:
  """Get today date

  Returns:
      dt.date: date object from datetime class
  """  
  return dt.datetime.now().date()

def days_before(days:int, date:dt.datetime = None) -> dt.date:
  """Get date of N days before the given date

  Args:
      days (int): N numbers of days
      date (dt.datetime, optional): initial date to subtract N days from. If not specified, it's using current date by default

  Returns:
      dt.date: N days before date
  """  
  if date is None:
    return (now() - dt.timedelta(days=days)).date() 
  return (date - dt.timedelta(days=days)).date()

def days_after(days:int, date:dt.datetime = None) -> dt.date:
  """Get date of N days after the given date

  Args:
      days (int): N numbers of days
      date (dt.datetime, optional): initial date to add N days to. If not specified, it's using current date by default

  Returns:
      dt.date: N days after date
  """  
  if date is None:
    return (now() + dt.timedelta(days=days)).date() 
  return (date + dt.timedelta(days=days)).date()

def yesterday() -> dt.date:
  """Yesterday date

  Returns:
      dt.date: yesterday date object
  """  
  return days_before(1)  

def tomorrow() -> dt.datetime:
  """Tomorrow date

  Returns:
      dt.datetime: tomorrow date object
  """  
  return days_after(1)

def currentHour() -> str:
  """Gets current hour in HH format. 
  For example: 00, 01, 02, ..., 12, 13, ... ,23

  Returns:
      str: 2 digits hour in 24 hour format
  """  
  return now().strftime('%H')

def currentMinute() -> str:
  """Gets current minute of the current hour

  Returns:
      str: minute
  """  
  return now().strftime('%M')

def str_to_date(text:str, dateFormat:str = '%Y-%m-%d') -> dt.date:
  """Convert string to date 

  Args:
      text (str): date string
      dateFormat (str, optional): Format of the given date string. Defaults to '%Y-%m-%d'.

  Returns:
      dt.datetime._date: date object
  """  
  return dt.datetime.strptime(text, dateFormat).date()

def str_to_datetime(text:str, dateTimeFormat:str = '%Y-%m-%d %H:%M:%S') -> dt.datetime:
  """Convert string to datetime

  Args:
      text (str): datetime string
      dateTimeFormat (str, optional): Format of the given datetime string. Defaults to '%Y-%m-%d %H:%M:%S'.

  Returns:
      dt.datetime: datetime object
  """  
  try:
    return dt.datetime.strptime(text, dateTimeFormat).replace(microsecond=0)
  except Exception as e:
    return None

def date_to_str(date:dt.date, date_format:str = '%Y-%m-%d') -> str:
  """Convert date object to string

  Args:
      date (dt.date): date object to convert to string
      date_format (str, optional): Format of output string date. Defaults to '%Y-%m-%d'.

  Returns:
      str: string of date
  """  
  return dt.datetime(date.year, date.month, date.day).strftime(date_format) 

def datetime_to_str(date:dt.datetime, date_format:str = '%Y-%m-%d %H:%M:%S') -> str:
  """Convert datetime object to string

  Args:
      date (dt.datetime): datetime object to convert to string
      date_format (str, optional): Format of output string date. Defaults to '%Y-%m-%d %H:%M:%S'.

  Returns:
      str: string of datetime
  """  
  return date.strftime(date_format) 



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='fp_th_di',
    packages=find_packages(include=[
        'fp_th_di'
    ]),
    version='1.0.3',
    description='Foodpanda Thailand Data & Insight team utility functions',
    author='Mathara Rojanamontien',
    author_email="mathara.rojanamontien@foodpanda.co.th",
    license='Delivery Hero (Thailand) Co., Ltd.',
    # install_requires=[],
    long_description=long_description,
    classifiers=(
        "Programming Language :: Python :: 3",
    ),
)